import { EventsHandler, IEventHandler } from '@nestjs/cqrs';
import { UserPrivilegeConfigUpdatedEvent } from '../../../entities/event/user-privilege-configuration-updated.event';
import { UserPrivilegeConfigurationService } from 'src/modules/user-related/user-privilege/data/service/user-privilege-configuration.service';
import { UserPrivilegeConfigurationHelper } from '../helpers/generate-user-privilege-configuration.helper';
import { UserPrivilegeConfigurationModel } from 'src/modules/user-related/user-privilege/data/models/user-privilege-configuration.model';

@EventsHandler(UserPrivilegeConfigUpdatedEvent)
export class UserPrivilegeConfigUpdateHandler
  implements IEventHandler<UserPrivilegeConfigUpdatedEvent>
{
  constructor(private dataService: UserPrivilegeConfigurationService) {}

  async handle(event: UserPrivilegeConfigUpdatedEvent) {
    const data = event.data.data;
    console.log(data.user_privilege_id);
    const configurations = await this.dataService.getManyByOptions({
      where: {
        user_privilege_id: data.user_privilege_id,
      },
    });
    const configs = UserPrivilegeConfigurationHelper.createConfigurations();

    const createdConfig = configs
      .filter(
        (base) =>
          !configurations.some((data) => {
            return base.menu_label == data.menu_label;
          }),
      )
      .map((item) => {
        return {
          ...item,
          user_privilege_id: data.user_privilege_id,
        };
      });

    const deletedConfig = configurations
      .filter(
        (base) =>
          !configs.some((data) => {
            return base.menu_label == data.menu_label;
          }),
      )
      ?.map((item) => item.id);

    const queryRunner = this.dataService
      .getRepository()
      .manager.connection.createQueryRunner();

    if (createdConfig.length) {
      await this.dataService.createBatch(
        queryRunner,
        UserPrivilegeConfigurationModel,
        createdConfig,
      );
    }

    if (deletedConfig.length) {
      await this.dataService.deleteByIds(
        queryRunner,
        UserPrivilegeConfigurationModel,
        deletedConfig,
      );
    }
  }
}
